File Init.c

// SCHEDA PICDEM2 PLUS
// Definizione della MAPPA DELLE PORTE, 
// con PIC18F242, 28 piedini, collegato attraverso il PORTB 0-3 al Display
// e di tutte le routine di INIZIALIZZAZIONE dell'Hardware...

/* direzione delle porte I/O -------------------------------------------*/

#define START_PORTA       0b00000000  // Valore iniziale USCITE PortA
#define DIR_PORT_A        0b11110001  //
                       /*   |||||||+---- PA(0) = Potenziometro
                            ||||||+----- PA(1) = LCD - Enable
                            |||||+------ PA(2) = LCD - R/W
                            ||||+------- PA(3) = LCD - RS
                            |||+-------- PA(4) = Switch S2 (negato)
                            ||+--------- PA(5) = N.c.
                            |+---------- PA(6) = 
                            +----------- PA(7) = 
                       */            
#define START_PORTB       0b00000000  // Valore iniziale USCITE PortB
#define DIR_PORT_B        0b11110000  //
                       /*   |||||||+---- PB(0) = LCD - Dato || Led
                            ||||||+----- PB(1) = LCD - Dato || Led
                            |||||+------ PB(2) = LCD - Dato || Led
                            ||||+------- PB(3) = LCD - Dato || Led
                            |||+-------- PB(4) = nc
                            ||+--------- PB(5) = nc
                            |+---------- PB(6) = ICD2
                            +----------- PB(7) = ICD2
                       */            
#define START_PORTC       0b00000000  // Valore iniziale USCITE PortC 
#define DIR_PORT_C        0b10111011   
                       /*   |||||||+---- PC(0) = nc
                            ||||||+----- PC(1) = nc
                            |||||+------ PC(2) = BUZZER
                            ||||+------- PC(3) = I2C-SCL
                            |||+-------- PC(4) = I2C-SDA
                            ||+--------- PC(5) = nc
                            |+---------- PC(6) = TX Seriale RS232
                            +----------- PC(7) = RX Seriale RS232
                       */            

// TASTI
#define S2                   PORTAbits.RA4
#define S3                   (TRISBbits.RB0=1, PORTAbits.RB0)

void InitPort(void)
  {
  // Inizializzazione PORTE del PIC
  PORTA = START_PORTA;
  TRISA = DIR_PORT_A;       
  ADCON1 = 0b00001110;	// Metto TUTTI gli ingressi di A in digitale
                        // tranne il primo, a cui  attaccato il POT
  PORTB = START_PORTB;
  TRISB = DIR_PORT_B;
  PORTC = START_PORTC;
  TRISC = DIR_PORT_C;
  }




File LCD.c

// ESkin Software PIC
// by Andrea Minghetti
// 24 settembre 2003

void LCD_SendByte(unsigned char dat)
{
 Delay100TCYx(20); 
 LCD_EN=1;  
  LCD_DB7=(dat & 0x80)>>7;
 LCD_DB6=(dat & 0x40)>>6;
 LCD_DB5=(dat & 0x20)>>5;
 LCD_DB4=(dat & 0x10)>>4;
 Delay100TCYx(30);  LCD_EN=0;  Delay100TCYx(20);
 
 LCD_EN=1; 
 LCD_DB7=(dat & 0x08)>>3;
 LCD_DB6=(dat & 0x04)>>2;
 LCD_DB5=(dat & 0x02)>>1;
 LCD_DB4=(dat & 0x01);
 Delay100TCYx(30);  LCD_EN=0;  Delay100TCYx(20);
}

void LCD_SendCommand(unsigned char dat)
{
 LCD_RS=0;  LCD_SendByte(dat);
}

void LCD_SendData(unsigned char dat)
{
 LCD_RS=1;  LCD_SendByte(dat); 
 Delay100TCYx(20);
}

void LCD_Locate(unsigned char row) // Versione DAVIDE (su tutto il display)
{
 LCD_RW=0; LCD_EN=0; LCD_RS=0;   // setto tutto basso...
 Delay100TCYx(20);   LCD_EN=1;   // aspetta e prepara l'enable alto:

 LCD_DB7=1;                      // Butta fuori il primo nibble di row
 if (row & 0x40) LCD_DB6 = 1;    // primo BIT sempre alto
            else LCD_DB6 = 0;
 if (row & 0x20) LCD_DB5 = 1;
            else LCD_DB5 = 0;
 if (row & 0x10) LCD_DB4 = 1;
            else LCD_DB4 = 0;
 Delay100TCYx(30); LCD_EN=0;
 Delay100TCYx(20); LCD_EN=1;     // colpo di Enable...
 
 if (row & 0x08) LCD_DB7 = 1;    // Butta fuori il secondo nibble di row
            else LCD_DB7 = 0;    // 
 if (row & 0x04) LCD_DB6 = 1;
            else LCD_DB6 = 0;
 if (row & 0x02) LCD_DB5 = 1;
            else LCD_DB5 = 0;
 if (row & 0x01) LCD_DB4 = 1;
            else LCD_DB4 = 0;
 Delay100TCYx(30); LCD_EN=0; Delay100TCYx(20);
}

void LCD_Clear(void)
{
 LCD_SendCommand(0x01);
 LCD_SendCommand(0x80);
}

void LCD_Init(void)
{
 LCD_RW=0;
 LCD_EN=0;
 LCD_RS=0;
 
 //0011
 Delay100TCYx(20); 
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0; 
 LCD_DB5=1;
 LCD_DB4=1;
 Delay100TCYx(200);
 LCD_EN=0;
 
 //0011
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=1;
 LCD_DB4=1; 
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0011
 Delay100TCYx(80);
 LCD_EN=1; 
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=1;
 LCD_DB4=1; 
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0010
 Delay100TCYx(80);
 LCD_EN=1;  
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=1;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0010
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=1;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //NF00
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=1;
 LCD_DB6=0;
 LCD_DB5=0;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0000
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=0;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0; 
 
 //0110
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=1;
 LCD_DB5=1;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0000
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=0;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //1100
 Delay100TCYx(80);
 LCD_EN=1; 
 LCD_DB7=1;
 LCD_DB6=1; 
 LCD_DB5=0; 
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0; 
 
 Delay100TCYx(80); 

}

void LCD_SendString(char * str)
{
 char dim;
 char i;
 unsigned char ch;
  
 dim=strlen(str);
 for (i=0;i<dim;i++)
 {
  ch=str[i];
  LCD_SendData(ch);
  Delay100TCYx(20); 
 } 
}



// ESkin Software PIC
// Matteo Nicolini
// 13 Marzo 2004

// libreria per la gestione del Display
//       SCHEDA PICDEM 2 PLUS


#define GODONE          ADCON0bits.GO 
#define ADON            ADCON0bits.ADON 
#define ADFM		ADCON1bits.ADFM

void ADC_Init(void)
	{
	ADFM=1;		//10 bit con intestazione a sinistra(2bit in ADRESH e gli 8 di ADRESL)
	}			

void ADC_Convert(void)
  	{  

// N.B.:La PORTA  gi stata settata con ingresso PA0 
// analogico nella funzione Init.
	
	ADON=1;        		//accendo l'ADconverter
	GODONE=1;			//faccio partire la conversione e
	while(GODONE){}		//aspetto che sia finita (a polling)
	ADON=0;
	}




File main.c

// ----------------------------------------------------------------
// --------  Funzionamento BASE della SCHEDA PIC-DEM2PLUS ---------
// --------         seminario del 6-3-2004 Cesena         ---------
// ----------------------------------------------------------------


#include <p18f452.h>
#include "Init.h"

// Variabili globali -------------------------------------------------------

 char cont = 0;   // Contatore 
 int  flag = 0;   // flag per il debounce del tasto.


// ----------------------------------------------------------------
//
//                 M  A  I  N            F  L  O  W 
//
// ----------------------------------------------------------------

void main(void)
  { 
  InitPort();      // Inizializzazione PORTE del PIC
  LCD_Init();      // Inizializzazione LCD   
  ADC_Init();	   // inizializzazione dell'ADC
  
  LCD_Clear();        // pulizia preventiva del LCD
  LCD_Locate(2);      // posizione del cursore: 2' carattere
  LCD_SendData('A');  // scrivo 'A': il cursore avanza automaticamente
  LCD_SendData('D');  // di un passo: scrivo 'D',
  LCD_SendData('C');  // scrivo 'C' e '='.
  LCD_SendData('=');  

  while (1)             // Ciclo main infinito.
    {
    if (!S2) 
       {
       ADC_Convert();	// la funzione ADC_Conv fa la conversione del valore
			// ANALOGICO all'ingresso PortA0 (potenziometro);
			// risultato e' contenuto nei registri del PIC
			// ADRESH (i 2 bit pi significativi, allineati a DX)
			// e ADRESL (gli altri 8 bit). 
       LCD_Locate(8);   // metto a posto il display:

       dato = ADRESH;	       // i due bit pi pesanti del ADConverter vengono
       chapp = dato +'0';      // sommati al carattere '0' in ChApp, che dunque 
       LCD_SendData(chapp);    // risulter '0','1','2' o '3'... e viene messo
                               // sul display dalla posizione 8
       dato = ADRESL / 16;     // il byte meno pesante del ADC viene diviso per
                               // 16: restano i 4 bit PESANTI, che sono convertiti
       if (dato>9)             // in esadecimale: se maggiore di 9 ci va una
          chapp = dato-10+'A'; // lettera, a partire dalla 'A'
       else                    // altrimenti ci va un numero, a partire da '0'
          chapp = dato + '0';  // comunque sia, ChApp viene messo a display
       LCD_SendData(chapp);    // nella posizione 9
                               
       dato = ADRESL % 16;     // il byte meno pesante del ADC viene diviso per 16
                               // e SI TIENE IL RESTO: sono i 4 bit LEGGERI, da mettere 
       if (dato>9)             // in esadecimale: se maggiore di 9 ci va una
          chapp = dato-10+'A'; // lettera, a partire dalla 'A'
       else                    // altrimenti ci va un numero, a partire da '0'
          chapp = dato + '0';  // comunque sia, ChApp viene messo a display
       LCD_SendData(chapp);    // nella posizione 9
       }   
    }
  }