File Init.c

// SCHEDA PICDEM2 PLUS
// Definizione della MAPPA DELLE PORTE, 
// con PIC18F242, 28 piedini, collegato attraverso il PORTB 0-3 al Display
// e di tutte le routine di INIZIALIZZAZIONE dell'Hardware...

/* direzione delle porte I/O -------------------------------------------*/

#define START_PORTA       0b00000000  // Valore iniziale USCITE PortA
#define DIR_PORT_A        0b11110001  //
                       /*   |||||||+---- PA(0) = Potenziometro
                            ||||||+----- PA(1) = LCD - Enable
                            |||||+------ PA(2) = LCD - R/W
                            ||||+------- PA(3) = LCD - RS
                            |||+-------- PA(4) = Switch S2 (negato)
                            ||+--------- PA(5) = N.c.
                            |+---------- PA(6) = 
                            +----------- PA(7) = 
                       */            
#define START_PORTB       0b00000000  // Valore iniziale USCITE PortB
#define DIR_PORT_B        0b11110000  //
                       /*   |||||||+---- PB(0) = LCD - Dato || Led
                            ||||||+----- PB(1) = LCD - Dato || Led
                            |||||+------ PB(2) = LCD - Dato || Led
                            ||||+------- PB(3) = LCD - Dato || Led
                            |||+-------- PB(4) = nc
                            ||+--------- PB(5) = nc
                            |+---------- PB(6) = ICD2
                            +----------- PB(7) = ICD2
                       */            
#define START_PORTC       0b00000000  // Valore iniziale USCITE PortC 
#define DIR_PORT_C        0b10111011   
                       /*   |||||||+---- PC(0) = nc
                            ||||||+----- PC(1) = nc
                            |||||+------ PC(2) = BUZZER
                            ||||+------- PC(3) = I2C-SCL
                            |||+-------- PC(4) = I2C-SDA
                            ||+--------- PC(5) = nc
                            |+---------- PC(6) = TX Seriale RS232
                            +----------- PC(7) = RX Seriale RS232
                       */            

// TASTI
#define S2                   PORTAbits.RA4
#define S3                   (TRISBbits.RB0=1, PORTAbits.RB0)

void InitPort(void)
  {
  // Inizializzazione PORTE del PIC
  PORTA = START_PORTA;
  TRISA = DIR_PORT_A;       
  ADCON1 = 0b00001110;	// Metto TUTTI gli ingressi di A in digitale
                        // tranne il primo, a cui  attaccato il POT
  PORTB = START_PORTB;
  TRISB = DIR_PORT_B;
  PORTC = START_PORTC;
  TRISC = DIR_PORT_C;
  }




File LCD.c

// ESkin Software PIC
// by Andrea Minghetti
// 24 settembre 2003

void LCD_SendByte(unsigned char dat)
{
 Delay100TCYx(20); 
 LCD_EN=1;  
  LCD_DB7=(dat & 0x80)>>7;
 LCD_DB6=(dat & 0x40)>>6;
 LCD_DB5=(dat & 0x20)>>5;
 LCD_DB4=(dat & 0x10)>>4;
 Delay100TCYx(30);  LCD_EN=0;  Delay100TCYx(20);
 
 LCD_EN=1; 
 LCD_DB7=(dat & 0x08)>>3;
 LCD_DB6=(dat & 0x04)>>2;
 LCD_DB5=(dat & 0x02)>>1;
 LCD_DB4=(dat & 0x01);
 Delay100TCYx(30);  LCD_EN=0;  Delay100TCYx(20);
}

void LCD_SendCommand(unsigned char dat)
{
 LCD_RS=0;  LCD_SendByte(dat);
}

void LCD_SendData(unsigned char dat)
{
 LCD_RS=1;  LCD_SendByte(dat); 
 Delay100TCYx(20);
}

void LCD_Locate(unsigned char row) // Versione DAVIDE (su tutto il display)
{
 LCD_RW=0; LCD_EN=0; LCD_RS=0;   // setto tutto basso...
 Delay100TCYx(20);   LCD_EN=1;   // aspetta e prepara l'enable alto:

 LCD_DB7=1;                      // Butta fuori il primo nibble di row
 if (row & 0x40) LCD_DB6 = 1;    // primo BIT sempre alto
            else LCD_DB6 = 0;
 if (row & 0x20) LCD_DB5 = 1;
            else LCD_DB5 = 0;
 if (row & 0x10) LCD_DB4 = 1;
            else LCD_DB4 = 0;
 Delay100TCYx(30); LCD_EN=0;
 Delay100TCYx(20); LCD_EN=1;     // colpo di Enable...
 
 if (row & 0x08) LCD_DB7 = 1;    // Butta fuori il secondo nibble di row
            else LCD_DB7 = 0;    // 
 if (row & 0x04) LCD_DB6 = 1;
            else LCD_DB6 = 0;
 if (row & 0x02) LCD_DB5 = 1;
            else LCD_DB5 = 0;
 if (row & 0x01) LCD_DB4 = 1;
            else LCD_DB4 = 0;
 Delay100TCYx(30); LCD_EN=0; Delay100TCYx(20);
}

void LCD_Clear(void)
{
 LCD_SendCommand(0x01);
 LCD_SendCommand(0x80);
}

void LCD_Init(void)
{
 LCD_RW=0;
 LCD_EN=0;
 LCD_RS=0;
 
 //0011
 Delay100TCYx(20); 
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0; 
 LCD_DB5=1;
 LCD_DB4=1;
 Delay100TCYx(200);
 LCD_EN=0;
 
 //0011
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=1;
 LCD_DB4=1; 
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0011
 Delay100TCYx(80);
 LCD_EN=1; 
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=1;
 LCD_DB4=1; 
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0010
 Delay100TCYx(80);
 LCD_EN=1;  
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=1;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0010
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=1;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //NF00
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=1;
 LCD_DB6=0;
 LCD_DB5=0;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0000
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=0;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0; 
 
 //0110
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=1;
 LCD_DB5=1;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //0000
 Delay100TCYx(80);
 LCD_EN=1;
 LCD_DB7=0;
 LCD_DB6=0;
 LCD_DB5=0;
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0;
 
 //1100
 Delay100TCYx(80);
 LCD_EN=1; 
 LCD_DB7=1;
 LCD_DB6=1; 
 LCD_DB5=0; 
 LCD_DB4=0;
 Delay100TCYx(80);
 LCD_EN=0; 
 
 Delay100TCYx(80); 

}

void LCD_SendString(char * str)
{
 char dim;
 char i;
 unsigned char ch;
  
 dim=strlen(str);
 for (i=0;i<dim;i++)
 {
  ch=str[i];
  LCD_SendData(ch);
  Delay100TCYx(20); 
 } 
}



// ESkin Software PIC
// Matteo Nicolini
// 13 Marzo 2004

// libreria per la gestione del Display
//       SCHEDA PICDEM 2 PLUS


#define GODONE          ADCON0bits.GO 
#define ADON            ADCON0bits.ADON 
#define ADFM		ADCON1bits.ADFM

void ADC_Init(void)
	{
	ADFM=1;		//10 bit con intestazione a sinistra(2bit in ADRESH e gli 8 di ADRESL)
	ADON=1;        		//accendo l'ADconverter
	}			

void ADC_Convert(void)
  	{               // N.B.:La PORTA  gi stata settata con ingresso PA0 
                        // analogico nella funzione Init.
	GODONE=1;	//faccio partire la conversione e
	while(GODONE){}	//aspetto che sia finita (a polling)
	}




// Inizializzazione del Timer 0 per 
// il contatore di FrameRate della 
// scheda di test PICDEM2+

// Davide Gennaretti e MAtteo Nicolini, Marzo 2004
 
//Inizializzazione per interrupt
void Init_TMR1 (void)
  {
  T1CON  =                  0b00110001;  // Setup Timer 1
                       // Bit 7..43..0
                       //     |||||||+-- TMR1ON: Accensione Timer 1
                       //     ||||||+--- TMR1CS: Clock Source
                       //     |||||+---- T1SINC: (negato): sincronizzazione al ExtClock
                       //     ||||+----- T1OSCEN: Oscillatore Timer1 On-Off
                       //     |||+------ T1CKPS0: Timer 1 Prescaler MSBIT : 1,2,4,8
                       //     ||+------- T1CKPS1: Timer 1 Prescaler LSBIT 
                       //     |+-------- nc
                       //     +--------- Read-Write mode a 16 Bit
                       // Setto con Ingresso sul CLock di sistema 
                       // 4 MHz diviso 4 = 1 MHz
                       // Attivo il prescaler da 8 --> 125KHz
                       // Setto il timer per contare 25000: cinque tick al secondo. 
  TMR1H = 0x9E;        // Devo settare 25.000 conteggi (=0x61A8) sul timer 1:  
  TMR1L = 0x57;        // visto che CONTA in AVANTI e scatta a 0xFFFF, 
                       // per contare 0x61A8 devo partire con 0xFFFF-0x61A8=0x9E57...

  INTCON =                  0b01000000;     //disable global and enable PIE
                       // Bit 7..43..0
                       //     |||||||+-- RBIF Port B on change Interrupt FLAG
                       //     ||||||+--- INT0IF INT0 (Pin Out) Interrupt FLAG
                       //     |||||+---- TMR0IF TMR0 Interrupt FLAG
                       //     ||||+----- RBIE Port B on change Interrupt Enable 
                       //     |||+------ INT0IE INT0 (Pin Out) Interrupt Enable 
                       //     ||+------- TMR0IE TMR0 Interrupt Enable 
                       //     |+-------- PIE Peripheral Interrupt Enable
                       //     +--------- GIE Global Interript Enable
                       
  INTCON2 =                 0b00000000;     // TMR0 disattivato
                       // Bit 7..43..0
                       //     |||||||+-- RBIP PortB Change Interrupt PRIORITY bit
                       //     ||||||+--- none
                       //     |||||+---- TMR0 Interrupt PRIORITY bit
                       //     ||||+----- none
                       //     |||+------ INTEDG2 External Interrupt 2 EDGE Select
                       //     ||+------- INTEDG1 External Interrupt 1 EDGE Select
                       //     |+-------- INTEDG0 External Interrupt 0 EDGE Select
                       //     +--------- RBPU weak pull up del PortB

  PIE1 =                    0b00000001;     //Attiva il TMR1
                       // Bit 7..43..0
                       //     |||||||+-- TMR1IE
                       //     ||||||+--- TMR2IE
                       //     |||||+---- CCP1IE
                       //     ||||+----- SSPIE
                       //     |||+------ TXIE
                       //     ||+------- RCIE
                       //     |+-------- ADIE
                       //     +--------- PSPIE (parallel slave port)

  IPR1 =                    0b00000001;   // Interrupt Priority Register:
                       // Bit 7..43..0
                       //     |||||||+-- TMR1IP
                       //     ||||||+--- TMR2IP
                       //     |||||+---- CCP1IP
                       //     ||||+----- SSPIP
                       //     |||+------ TXIP
                       //     ||+------- RCIP
                       //     |+-------- ADIP
                       //     +--------- PSPIP (parallel slave port)

  RCONbits.IPEN = 0;   // disable priority levels
  }







File main.c

// ----------------------------------------------------------------
// --------  Funzionamento BASE della SCHEDA PIC-DEM2PLUS ---------
// --------         seminario del 6-3-2004 Cesena         ---------
// ----------------------------------------------------------------


#include <p18f242.h>
#include <string.h>
#include <delays.h>
#include <timers.h>
#include "RS232.h"
#include "I2C.h"
#include "lcd.h"
#include "Init.h"
#include "ADC.h"
#include "timer1.h"


// ******************** PROTOTIPI FUNZIONI  ******************************

// Solo interrupt... le altre sono nei file header !------------------------
void isr (void);

#pragma code InterruptVectorHigh = 0x08
void
InterruptVectorHigh (void)
{
  _asm
    goto isr //jump to interrupt routine
  _endasm
}


// Variabili globali -------------------------------------------------------

unsigned char dato;		    //Variabili utilizzate per mostrare
unsigned char chapp;		    //la conversione dei dati dell'ADC

unsigned char cont = 0;             // contatore
unsigned int  flag = 0;             // flag per il DEBOUNCE
    
unsigned char FR_Cnt = 0;           // Main Counter per il frame Rate
unsigned char FR_Medio = 0;         // Visualizzatore del Frame Rate Medio.
unsigned char Secondi = 0;          // Contasecondi


// ----------------------------------------------------------------
//
//                 M  A  I  N            F  L  O  W 
//
// ----------------------------------------------------------------

# pragma cod

void main(void)
  {
  Init_Port();  // Inizializzazione PORTE del PIC
  LCD_Init();   // Start del LCD
  ADC_Init();	// inizializzazione dell'ADC
  Init_TMR1();  // inizializzazione Timer 1

  LCD_Clear();                     // pulizia preventiva del LCD
  LCD_Locate(2);
  LCD_SendData('A');
  LCD_SendData('D');
  LCD_SendData('C');
  LCD_SendData('=');  
                               // Avvio di tutti i servizi di interrupt
  INTCONbits.GIE = 1;          // alla fine di tutte le inizializzazioni!
       
  while (1)
    {
    ADC_Convert();	// la funzione ADC_Conv fa la conversione del valore
	    	        // ANALOGICO all'ingresso PortA0 (potenziometro);
			// risultato e' contenuto nei registri del PIC
			// ADRESH (i 2 bit pi significativi, allineati a DX)
			// e ADRESL (gli altri 8 bit). 
    LCD_Locate(8);   	// metto a posto il display:

    dato = ADRESH;	    // i due bit pi pesanti del ADConverter vengono
    chapp = dato +'0';      // sommati al carattere '0' in ChApp, che dunque 
    LCD_SendData(chapp);    // risulter '0','1','2' o '3'... e viene messo
                            // sul display dalla posizione 8
    dato = ADRESL / 16;     // il byte meno pesante del ADC viene diviso per
                            // 16: restano i 4 bit PESANTI, che sono convertiti
    if (dato>9)             // in esadecimale: se maggiore di 9 ci va una
       chapp = dato-10+'A'; // lettera, a partire dalla 'A'
    else                    // altrimenti ci va un numero, a partire da '0'
       chapp = dato + '0';  // comunque sia, ChApp viene messo a display
    LCD_SendData(chapp);    // nella posizione 9
                             
    dato = ADRESL % 16;     // il byte meno pesante del ADC viene diviso per 16
                            // e SI TIENE IL RESTO: sono i 4 bit LEGGERI, da mettere 
    if (dato>9)             // in esadecimale: se maggiore di 9 ci va una
       chapp = dato-10+'A'; // lettera, a partire dalla 'A'
    else                    // altrimenti ci va un numero, a partire da '0'
       chapp = dato + '0';  // comunque sia, ChApp viene messo a display
    LCD_SendData(chapp);    // nella posizione 9
	
    LCD_Locate(74);         // SCRIVE il numeri 'secondi' sul display
    LCD_SendNum(Secondi);   // in posizione 64 (=riga2) + 10
                            // ma chi scrive il valore di 'secondi'???
	   
    }  // END while infinito
  }    // END main 



//----------------------------------------------------------------------------
// High priority interrupt routine

# pragma interrupt isr

void isr()
  {
  INTCON =  0b01000000;   // Risetta INTCON per sicurezza
  if (PIR1bits.TMR1IF)    // se scatta il timer 1 
    {                     // PRIMA ripristino il TIMER: 
    TMR1H = 0x9E; //61;   // Setto 25.000 (=0x61A8) sul timer 1: poi
    TMR1L = 0x57; //A8;   // incremento di due i decimi di secondo
    Secondi ++;
    Secondi ++;
    PIR1bits.TMR1IF = 0;  // resetto il flag dell'interrupt: 
    }
  }                       // end isr()

   